/*
 * calc.hpp
 *
 *  Created on: 26.11.2008
 */

#ifndef CALC_HPP_
#define CALC_HPP_



#include <stack>
#include <sstream>

typedef enum {
  NOOP,
  PLUS,
  MINUS,
  MULTIPLY,
  DIVISION,
  SQUARE,
  MOD,
  NEGATE,
  DUP,
  DROP
} Operation;

class Calculator {
public:
	Calculator(std::stack<int>* _operands):operands(_operands) {
	}
	void compute(Operation op) {
		int op1 = 0, op2 = 0;
		int result = 0;
		switch (op) {
		case PLUS:
			get2operands(op1,op2);
			result = op1 + op2;
			operands->push(result);
			break;
		case MINUS:
			get2operands(op2,op1);
			result = op1 - op2;
			operands->push(result);
			break;
		case MULTIPLY:
			get2operands(op2,op1);
			result = op1 * op2;
			operands->push(result);
			break;
		case DIVISION:
			get2operands(op2,op1);
			if (op2 == 0) result = 0;
			else result = op1 / op2;
			operands->push(result);
			break;
		case MOD:
			get2operands(op2,op1);
			result = op1 % op2;
			operands->push(result);
			break;
		case SQUARE:
			get1operand(op1);
			result = op1 * op1;
			operands->push(result);
			break;
		case NEGATE:
			get1operand(op1);
			result = -op1;
			operands->push(result);
			break;
		}
	}

private:
	void get2operands(int& a, int& b) {
		get1operand(a);
		get1operand(b);
	}

	void get1operand(int& a) {
		assert(!operands->empty());
		a = operands->top();
		operands->pop();
	}

	std::stack<int>* operands;
};


class CalculatorInterface {
public:
	CalculatorInterface(std::istream& _input, std::ostream& _output):
		input(_input),
		output(_output),
		operands(),
		calc(&operands) {
	}

	void run() {
		while (true) {
			std::string s;
			input >> s;
			if (s=="plus") {
				performOp(PLUS);
			} else if (s == "minus") {
				performOp(MINUS);
			} else if (s == "div") {
				performOp(DIVISION);
			} else if (s == "multiply") {
				performOp(MULTIPLY);
			} else if (s == "mod") {
				performOp(MOD);
			} else if (s == "square") {
				performOp(SQUARE);
			} else if (s == "negate") {
				performOp(NEGATE);
			} else if (s=="") {
				printStack();
			} else if (s=="QUIT") { return;
			} else {
				// s must be a number
				std::stringstream buf;
				buf << s;
				int num;
				buf >> num;
				operands.push(num);
			}
		}
	}

private:
	void performOp(Operation op) {
		printStack();
		calc.compute(op);
		output << " = ";
		printStack();
		output << std::endl;
		std::flush(output);
	}

	void printStack() {
//		for(int i = 0; i < operands->size(); i++) {
//			output << *operands[i] << " ";
//		}
	}

	std::istream& input;
	std::ostream& output;
	std::stack<int> operands;
	Calculator calc;
};

//int main(){
//    CalculatorInterface ci(std::cin, std::cout);
//    ci.run();
//}

#endif /* CALC_HPP_ */
